using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;
using Microsoft.Phone.Tasks;

namespace LauncherExample
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        SpriteFont font;
        string _currentString = "Waiting for something...";
        private Rectangle[] ButtonList = new Rectangle[16];
        private const int ButtonSize = 100;
        Texture2D _gray;
        Texture2D _chosenPhoto;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            graphics.PreferredBackBufferHeight = 800;
            graphics.PreferredBackBufferWidth = 480;

            // Use some gestures
            TouchPanel.EnabledGestures = GestureType.Tap;
            for (int i = 0; i < ButtonList.Length; i++)
            {
                int x = (i % 4);
                int y = (i / 4);

                ButtonList[i] = new Rectangle((10 * (x + 1)) + (x * ButtonSize), 
                    25 + (10 * (y + 1)) + (y * ButtonSize), ButtonSize, ButtonSize);
            }

            // Frame rate is 30 fps by default for Windows Phone.
            TargetElapsedTime = TimeSpan.FromTicks(333333);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            _gray = new Texture2D(GraphicsDevice, 1, 1);
            _gray.SetData<Color>(new Color[] { Color.Gray });

            // TODO: use this.Content to load your game content here
            font = Content.Load<SpriteFont>("Text");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            while (TouchPanel.IsGestureAvailable)
            {
                GestureSample gesture = TouchPanel.ReadGesture();
                switch (GetButton(gesture))
                {
                    case 0: // Phone caller
                        {
                            PhoneCallTask phone = new PhoneCallTask();
                            phone.DisplayName = "Jenny";
                            phone.PhoneNumber = "1-800-867-5309";
                            phone.Show();
                        }
                        break;
                    case 1: // Web task
                        {
                            WebBrowserTask web = new WebBrowserTask();
                            web.URL = "http://developer.windowsphone.com";
                            web.Show();
                        }
                        break;
                    case 2: // Send email
                        {
                            EmailComposeTask email = new EmailComposeTask();
                            email.Subject = "This is my game!";
                            email.Body = "This game rocks.";
                            email.Show();
                        }
                        break;
                    case 3: // SMS Compose task
                        {
                            SmsComposeTask sms = new SmsComposeTask();
                            sms.Body = "Sending you a text from my awesome game";
                            sms.To = "John Doe";
                            sms.Show();
                        }
                        break;
                    case 4: // MarketplaceDetailTask Task
                        {
                            MarketplaceDetailTask market = new MarketplaceDetailTask();
                            market.ContentType = MarketplaceContentType.Applications;
                            market.Show();
                        }
                        break;
                    case 5: // MarketplaceHubTask Task
                        {
                            MarketplaceHubTask market = new MarketplaceHubTask();
                            market.ContentType = MarketplaceContentType.Applications;
                            market.Show();
                        }
                        break;
                    case 6: // MarketplaceReviewTask Task
                        {
                            MarketplaceReviewTask market = new MarketplaceReviewTask();
                            market.Show();
                        }
                        break;
                    case 7: // MarketplaceSearchTask Task
                        {
                            MarketplaceSearchTask market = new MarketplaceSearchTask();
                            market.ContentType = MarketplaceContentType.Applications;
                            market.SearchTerms = "xna";
                            market.Show();
                        }
                        break;
                    case 8: // Search Task
                        {
                            SearchTask search = new SearchTask();
                            search.SearchQuery = "XNA Game Studio";
                            search.Show();
                        }
                        break;
                    case 9: // MediaPlayerLauncher Task
                        {
                            MediaPlayerLauncher media = new MediaPlayerLauncher();
                            media.Controls = MediaPlaybackControls.All;
                            media.Location = MediaLocationType.Install;
                            media.Media = new Uri("samplemovie.wmv", UriKind.Relative);
                            media.Show();
                        }
                        break;

                    case 10: // Photochooser Task
                        {
                            PhotoChooserTask photo = new PhotoChooserTask();
                            photo.ShowCamera = true;
                            photo.Completed += new EventHandler<PhotoResult>((o, result) =>
                            {
                                if (result.TaskResult == TaskResult.OK)
                                {
                                    _currentString = "Current photo original filename was " +
                                        result.OriginalFileName;
                                    _chosenPhoto = Texture2D.FromStream(GraphicsDevice,
                                        result.ChosenPhoto);
                                }
                                else if (result.TaskResult == TaskResult.Cancel)
                                {
                                    _currentString = "Choosing photo was cancelled";
                                }
                            });
                            photo.Show();
                        }
                        break;
                    case 11: // CameraCaptureTask Task
                        {
                            CameraCaptureTask camera = new CameraCaptureTask();
                            camera.Completed += new EventHandler<PhotoResult>((o, result) =>
                            {
                                if (result.TaskResult == TaskResult.OK)
                                {
                                    _currentString = "Camera pictures have no original filename " +
                                        result.OriginalFileName;
                                    _chosenPhoto = Texture2D.FromStream(GraphicsDevice,
                                        result.ChosenPhoto);
                                }
                                else if (result.TaskResult == TaskResult.Cancel)
                                {
                                    _currentString = "Taking picture was cancelled";
                                }
                            });
                            camera.Show();
                        }
                        break;
                    case 12: // Email Chooser
                        {
                            EmailAddressChooserTask email = new EmailAddressChooserTask();
                            email.Completed += new EventHandler<EmailResult>((o, result) =>
                            {
                                if (result.TaskResult == TaskResult.OK)
                                {
                                    _currentString = "Email address is " + result.Email;
                                }
                                else if (result.TaskResult == TaskResult.Cancel)
                                {
                                    _currentString = "Choosing email was cancelled";
                                }
                            });
                            email.Show();
                        }
                        break;
                    case 13: // Save email task
                        {
                            SaveEmailAddressTask save = new SaveEmailAddressTask();
                            save.Email = "John.Doe@fakecompany.org";
                            save.Completed += new EventHandler<TaskEventArgs>((o, result) =>
                            {
                                if (result.TaskResult == TaskResult.OK)
                                {
                                    _currentString = "Successfully saved contact email.";
                                }
                                else if (result.TaskResult == TaskResult.Cancel)
                                {
                                    _currentString = "Cancelled saving contact email.";
                                }
                            });
                            save.Show();
                        }
                        break;
                    case 14: // Phone Number Chooser
                        {
                            PhoneNumberChooserTask phone = new PhoneNumberChooserTask();
                            phone.Completed += new EventHandler<PhoneNumberResult>((o, result) =>
                            {
                                if (result.TaskResult == TaskResult.OK)
                                {
                                    _currentString = "Phone number is " + result.PhoneNumber;
                                }
                                else if (result.TaskResult == TaskResult.Cancel)
                                {
                                    _currentString = "Choosing phone number was cancelled";
                                }
                            });
                            phone.Show();
                        }
                        break;
                    case 15: // Save phone number task
                        {
                            SavePhoneNumberTask save = new SavePhoneNumberTask();
                            save.PhoneNumber = "1-800-867-5309";
                            save.Completed += new EventHandler<TaskEventArgs>((o, result) =>
                            {
                                if (result.TaskResult == TaskResult.OK)
                                {
                                    _currentString = "Successfully saved contact email.";
                                }
                                else if (result.TaskResult == TaskResult.Cancel)
                                {
                                    _currentString = "Cancelled saving contact email.";
                                }
                            });
                            save.Show();
                        }
                        break;
                    default:
                        // Do nothing
                        break;
                }
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// Get which button was tapped, if any
        /// </summary>
        int GetButton(GestureSample gesture)
        {
            for (int i = 0; i < ButtonList.Length; i++)
            {
                if (ButtonList[i].Contains((int)gesture.Position.X, 
                    (int)gesture.Position.Y))
                {
                    return i;
                }
            }
            return -1;
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            spriteBatch.Begin();
            foreach (Rectangle r in ButtonList)
            {
                Rectangle border = r;
                border.Inflate(1, 1);
                spriteBatch.Draw(_gray, border, Color.Black);
                spriteBatch.Draw(_gray, r, Color.White);
            }

            // Draw the chosen photo if it exists
            if (_chosenPhoto != null)
            {
                spriteBatch.DrawString(font, "Current photo:", 
                    new Vector2(20, 500), Color.White);
                Vector2 v = font.MeasureString("Current photo:");
                spriteBatch.Draw(_chosenPhoto, new Rectangle(20, 
                    (int)v.Y + 505, 200, 200), Color.White);
            }
            spriteBatch.DrawString(font, _currentString, 
                new Vector2(10, 5), Color.White);
            spriteBatch.End();
        }
    }
}
