float4x4 World;
float4x4 View;
float4x4 Projection;

float3 AmbientLightColor;
float3 DiffuseColor;
float4 SpecularColorPower;
float3 EmissiveColor;

float3 LightPosition;
float LightRange;
float3 DiffuseLightColor;
float3 SpecularLightColor;

float3 CameraPosition;

struct VertexShaderInput
{
    float4 Position : POSITION0;
    float3 Normal   : NORMAL;
};

struct VertexShaderOutput
{
    float4 Position : POSITION0;
    float3 Normal   : TEXCOORD0;
    float3 View     : TEXCOORD1;
    float3 WorldPos : TEXCOORD2;
};

VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
    VertexShaderOutput output;

    float4 worldPosition = mul(input.Position, World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);
    output.Normal = mul(input.Normal, World);
    output.View = CameraPosition - worldPosition;
    output.WorldPos = worldPosition;

    return output;
}

float4 PixelShaderFunction(VertexShaderOutput input) : COLOR0
{
    // Normalize the interpolated normal and view
    float3 normal = normalize(input.Normal);
    float3 view = normalize(input.View);

    // Direction from pixel to the light
    float3 light = LightPosition - input.WorldPos;

    // Attenuation
    float attenuation = saturate(1 - dot(light / LightRange, light / LightRange));

    // Normalize the light direction
    light = normalize(light);

    // Store the final color of the pixel
    float3 finalColor = float3(0, 0, 0);

    // Start with ambient light color
    float3 diffuse = AmbientLightColor;

    // Calculate diffuse lighting
    float NdotL = saturate(dot(normal, light));
    diffuse += NdotL * DiffuseLightColor * attenuation;

    // Calculate half vector
    float3 half = normalize(view + light);

    // Calculate N * H
    float NdotH = saturate(dot(normal, half));

    // Calculate specular using Blinn-Phong
    float specular = 0;
    if (NdotL != 0)
        specular += pow(NdotH, SpecularColorPower.w) * SpecularLightColor * attenuation;

    // Add in diffuse color value
    finalColor +=  DiffuseColor * diffuse;

    // Add in specular color value
    finalColor +=  SpecularColorPower.xyz * specular;

    // Add in emissive color
    finalColor += EmissiveColor;

    return float4(finalColor, 1);
}

technique Technique1
{
    pass Pass1
    {
        // TODO: set renderstates here.

        VertexShader = compile vs_2_0 VertexShaderFunction();
        PixelShader = compile ps_2_0 PixelShaderFunction();
    }
}
