using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace ContentImporter
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Level mylevel;
        Matrix view;
        Matrix proj;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            view = Matrix.CreateLookAt(new Vector3(0, 10, 150), Vector3.Zero, Vector3.Up);
            proj = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, 
                GraphicsDevice.Viewport.AspectRatio,
                1.0f, 1000.0f);
            mylevel = Content.Load<Level>("mylevel");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            spriteBatch.Begin();
            spriteBatch.Draw(mylevel.Background, GraphicsDevice.Viewport.Bounds, Color.White);
            spriteBatch.End();
            foreach (MyLevelObject myObject in mylevel.LevelObjects)
            {
                foreach (ModelMesh mesh in myObject.Model.Meshes)
                {
                    foreach (BasicEffect effect in mesh.Effects)
                    {
                        effect.EnableDefaultLighting();
                    }
                }
                myObject.Model.Draw(Matrix.CreateTranslation(
                        myObject.Position) * Matrix.CreateFromYawPitchRoll(
                        myObject.Rotation.X, myObject.Rotation.Y,
                        myObject.Rotation.Z), view, proj);
            }
        }
    }

    public class MyLevelObject
    {
        public Model Model;
        public Vector3 Position;
        public Vector3 Rotation;
    }

    public class Level
    {
        public List<MyLevelObject> LevelObjects;
        public Texture2D Background;
    }

    public class LevelContentReader : ContentTypeReader<Level>
    {
        protected override Level Read(ContentReader input, Level existingInstance)
        {
            Level level = new Level();
            level.Background = input.ReadObject<Texture2D>();
            int count = input.ReadInt32();
            level.LevelObjects = new List<MyLevelObject>(count);
            for (int i = 0; i < count; i++)
            {
                MyLevelObject obj = new MyLevelObject();
                obj.Model = input.ReadObject<Model>();
                obj.Position = input.ReadVector3();
                obj.Rotation = input.ReadVector3();
                level.LevelObjects.Add(obj);
            }

            return level;
        }
    }
}
