﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.GamerServices;

namespace CustomAvatarAnimation
{
    // The class that plays the custom animation
    public class AvatarCustomAnimation : CustomAvatarAnimationData, IAvatarAnimation
    {
        public ReadOnlyCollection<Matrix> BoneTransforms
        {
            get { return boneTransforms; }
        }
        private ReadOnlyCollection<Matrix> boneTransforms;
        Matrix[] avatarBoneTransforms = new Matrix[AvatarRenderer.BoneCount];

        public TimeSpan CurrentPosition
        {
            get
            {
                return currentPosition;
            }
            set
            {
                currentPosition = value;

                // Reset the animation and update the current transfroms
                currentKeyframe = 0;
                Update(TimeSpan.Zero, false);
            }
        }
        private TimeSpan currentPosition = TimeSpan.Zero;
        private int currentKeyframe = 0;

        public AvatarExpression Expression
        {
            get { return avatarExpression; }
        }
        AvatarExpression avatarExpression = new AvatarExpression();

        public AvatarCustomAnimation(CustomAvatarAnimationData animationData) : 
            base(animationData.Name, animationData.Length, animationData.Keyframes)
        {
            // Initialize bone transforms
            for (int i = 0; i < AvatarRenderer.BoneCount; i++)
            {
                avatarBoneTransforms[i] = Matrix.Identity;
            }
            boneTransforms = new ReadOnlyCollection<Matrix>(avatarBoneTransforms);

            // Populate the transfroms for the first frame
            Update(TimeSpan.Zero, false);
        }


        public void Update(TimeSpan timeSpan, bool loop)
        {
            // Incrament the curret time
            currentPosition += timeSpan;

            // Check current time against the length
            if (currentPosition > Length)
            {
                if (loop)
                {
                    // Find the right time in the new loop iteration
                    while (currentPosition > Length)
                    {
                        currentPosition -= Length;
                    }
                    // Set the keyframe to 0.
                    currentKeyframe = 0;
                }
                else
                {
                    // If the animation is not looping, 
                    // then set the time to the end of the animation.
                    currentPosition = Length;
                }
            }
            // Check to see if we are less than zero
            else if (currentPosition < TimeSpan.Zero)
            {
                if (loop)
                {
                    // If the animation is looping, 
                    // then find the right time in the new loop iteration
                    while (currentPosition < TimeSpan.Zero)
                    {
                        currentPosition += Length;
                    }
                    // Set the keyframe to the last keyframe
                    currentKeyframe = Keyframes.Count - 1;
                }
                else
                {
                    // If the animation is not looping, 
                    // then set the time to the beginning of the animation.
                    currentPosition = TimeSpan.Zero;
                }
            }

            // Update the bone transforms based on the current time.
            UpdateBoneTransforms(timeSpan >= TimeSpan.Zero);
        }


        /// <summary>
        /// Updates the transforms with the correct keyframes based on the current time.
        /// </summary>
        /// <param name="playingForward">
        /// If true, the animation is playing forward; otherwise, it is playing backwards
        /// </param>
        private void UpdateBoneTransforms(bool playingForward)
        {
            if (playingForward)
            {
                while (currentKeyframe < Keyframes.Count)
                {
                    // Get the current keyframe
                    Keyframe keyframe = Keyframes[currentKeyframe];

                    // Stop when we've read up to the current time.
                    if (keyframe.Time > currentPosition)
                        break;

                    // Apply the current keyframe's transform to the bone array.
                    avatarBoneTransforms[keyframe.BoneIndex] = keyframe.Transform;

                    // Move the current keyframe forward.
                    currentKeyframe++;
                }
            }
            else
            {
                while (currentKeyframe >= 0)
                {
                    // Get the current keyframe
                    Keyframe keyframe = Keyframes[currentKeyframe];

                    // Stop when we've read back to the current time.
                    if (keyframe.Time < currentPosition)
                        break;

                    // Apply the current keyframe's transform to the bone array.
                    avatarBoneTransforms[keyframe.BoneIndex] = keyframe.Transform;

                    // Move the current keyframe backwards.
                    currentKeyframe--;
                }
            }
        }
    }
}
