using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;
using Microsoft.Devices.Sensors;

namespace AccelerometerInput
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Accelerometer accelerometer;

        Texture2D spriteTexture;
        Vector2 spritePosition;
        Vector2 spriteVelocity;
        float spriteSpeed = 50.0f;
        int screenWidth = 480;
        int screenHeight = 800;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            // Frame rate is 30 fps by default for Windows Phone.
            TargetElapsedTime = TimeSpan.FromTicks(333333);

            // Pre-autoscale settings.
            graphics.PreferredBackBufferWidth = 480;
            graphics.PreferredBackBufferHeight = 800;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // Create instance of the AccelerometerSensor
            accelerometer = new Accelerometer();
            accelerometer.ReadingChanged += new EventHandler<AccelerometerReadingEventArgs>(accelerometer_ReadingChanged);
            try
            {
                accelerometer.Start();
            }
            catch (AccelerometerFailedException)
            {
                // Accelerometer not supported on this platform
                // We are going to exit.
                Exit();
            }

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            spriteTexture = Content.Load<Texture2D>("XnaLogo");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        protected override void OnExiting(object sender, EventArgs args)
        {
            try
            {
                accelerometer.Stop();
            }
            catch (AccelerometerFailedException)
            {
                // Do nothing since we are already exiting
            }

            base.OnExiting(sender, args);
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // Update sprite position
            spritePosition += spriteVelocity * (float)gameTime.ElapsedGameTime.TotalSeconds;

            // Dampen velocity
            spriteVelocity *= 1 - (float)gameTime.ElapsedGameTime.TotalSeconds;

            // Check sprite bounds
            if (spritePosition.X < 0)
            {
                spritePosition.X = 0;
                spriteVelocity.X = 0;
            }
            else if (spritePosition.X > screenWidth - spriteTexture.Width)
            {
                spritePosition.X = screenWidth - spriteTexture.Width;
                spriteVelocity.X = 0;
            }
            if (spritePosition.Y < 0)
            {
                spritePosition.Y = 0;
                spriteVelocity.Y = 0;
            }
            else if (spritePosition.Y > screenHeight - spriteTexture.Height)
            {
                spritePosition.Y = screenHeight - spriteTexture.Width;
                spriteVelocity.Y = 0;
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // Draw the sprite
            spriteBatch.Begin();
            spriteBatch.Draw(spriteTexture, spritePosition, Color.White);
            spriteBatch.End();

            base.Draw(gameTime);
        }

        public void accelerometer_ReadingChanged(object sender, AccelerometerReadingEventArgs e)
        {
            Vector3 accelerometerReading;
            accelerometerReading.X = (float)e.X;
            accelerometerReading.Y = (float)e.Y;
            accelerometerReading.Z = (float)e.Z;

            // Update velocity
            spriteVelocity.X += accelerometerReading.X * spriteSpeed;
            spriteVelocity.Y += accelerometerReading.Y * spriteSpeed;
        }
    }
}
