using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;

namespace GamePadInput
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Texture2D spriteTexture;
        Vector2 spritePosition = new Vector2(200, 200);
        Vector2 spriteCenter;
        float spriteSpeed = 100f;
        GamePadState lastGamePadState;
        float rotation;
        float scale = 1.0f;
        SpriteEffects flipOptions;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            spriteTexture = Content.Load<Texture2D>("XnaLogo");
            spriteCenter.X = spriteTexture.Width / 2.0f;
            spriteCenter.Y = spriteTexture.Height / 2.0f;
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // Read the current gamepad state
            GamePadState currentGamePadState = GamePad.GetState(PlayerIndex.One);

            // Move the sprite based on the left thumb stick
            spritePosition.X += spriteSpeed * 
                                currentGamePadState.ThumbSticks.Left.X * 
                                (float)gameTime.ElapsedGameTime.TotalSeconds;
            spritePosition.Y -= spriteSpeed *
                                currentGamePadState.ThumbSticks.Left.Y *
                                (float)gameTime.ElapsedGameTime.TotalSeconds;

            // Flip the sprite horizontally if the A button is pressed
            if (currentGamePadState.Buttons.A == ButtonState.Pressed &&
                lastGamePadState.Buttons.A == ButtonState.Released)
            {
                flipOptions ^= SpriteEffects.FlipHorizontally;
            }
            // Flip the sprite vertically if the X button is pressed
            if (currentGamePadState.Buttons.X == ButtonState.Pressed &&
                lastGamePadState.Buttons.X == ButtonState.Released)
            {
                flipOptions ^= SpriteEffects.FlipVertically;
            }

            // Set rotation based on trigger positions
            rotation = (-currentGamePadState.Triggers.Left +
                         currentGamePadState.Triggers.Right) *
                         MathHelper.Pi;

            // Set scale with DPad Up and Down buttons
            if (currentGamePadState.DPad.Up == ButtonState.Pressed)
                scale += (float)gameTime.ElapsedGameTime.TotalSeconds;
            else if (currentGamePadState.DPad.Down == ButtonState.Pressed)
                scale -= (float)gameTime.ElapsedGameTime.TotalSeconds;

            // Save the last frames state
            lastGamePadState = currentGamePadState;

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();
            spriteBatch.Draw(spriteTexture, spritePosition, null, Color.White,
                             rotation, spriteCenter, scale, flipOptions, 0);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
