using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;

namespace GestureInput
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        SpriteFont spriteFont;
        List<GestureSample> gestures = new List<GestureSample>();

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.SupportedOrientations = DisplayOrientation.Portrait;
            graphics.IsFullScreen = true;
            graphics.PreferredBackBufferWidth = 480;
            graphics.PreferredBackBufferHeight = 800;

            Content.RootDirectory = "Content";

            // Frame rate is 30 fps by default for Windows Phone.
            TargetElapsedTime = TimeSpan.FromTicks(333333);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // Enable the gestures we want to recieve 
            TouchPanel.EnabledGestures = GestureType.DoubleTap | GestureType.DragComplete | 
                                         GestureType.Flick | GestureType.FreeDrag | 
                                         GestureType.Hold | GestureType.HorizontalDrag |
                                         GestureType.Pinch | GestureType.PinchComplete |
                                         GestureType.Tap | GestureType.VerticalDrag;

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            spriteFont = Content.Load<SpriteFont>("SpriteFont1");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // Loop while there are still gestures to read
            while (TouchPanel.IsGestureAvailable)
            {
                // Add each to the stack
                gestures.Add(TouchPanel.ReadGesture());
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();
            Vector2 textPosition = new Vector2(5, 5);

            // Loop over all of the gestures and draw their information to the screen
            for (int i = gestures.Count-1; i >= 0; i--)
            {
                // Draw all of the properties of the gesture
                GestureSample gesture = gestures[i];
                spriteBatch.DrawString(spriteFont, "GestureType: " + gesture.GestureType.ToString(), textPosition, Color.White);
                textPosition.Y += 25;
                spriteBatch.DrawString(spriteFont, "Position: " + gesture.Position.ToString(), textPosition, Color.White);
                textPosition.Y += 25;
                spriteBatch.DrawString(spriteFont, "Position2: " + gesture.Position2.ToString(), textPosition, Color.White);
                textPosition.Y += 25;
                spriteBatch.DrawString(spriteFont, "Delta: " + gesture.Delta.ToString(), textPosition, Color.White);
                textPosition.Y += 25;
                spriteBatch.DrawString(spriteFont, "Delta2: " + gesture.Delta2.ToString(), textPosition, Color.White);
                textPosition.Y += 25;
                spriteBatch.DrawString(spriteFont, "Timestamp: " + gesture.Timestamp.ToString(), textPosition, Color.White);
                textPosition.Y += 40;
            }
            spriteBatch.End();

            // Remove any gestures that wont print on the screen
            if (gestures.Count > 5)
                gestures.RemoveRange(0, gestures.Count - 5);

            base.Draw(gameTime);
        }
    }
}
