using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;

namespace DynamicPlayback
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        const int SampleRate = 48000;
        DynamicSoundEffectInstance dynamicSoundEffectInstance;
        byte[] buffer;
        int bufferSize;

        // Frequency to generate
        double frequency = 200;
        // Counter to mark where we are in a wave
        int totalTime = 0;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // Create new dynamic sound effect and start playback
            dynamicSoundEffectInstance = new DynamicSoundEffectInstance(SampleRate, AudioChannels.Mono);
            dynamicSoundEffectInstance.BufferNeeded += new EventHandler<EventArgs>(OnBufferNeeded);
            dynamicSoundEffectInstance.Play();

            // Calculate the buffer size to hold 1 second
            bufferSize = dynamicSoundEffectInstance.GetSampleSizeInBytes(TimeSpan.FromSeconds(1));
            buffer = new byte[bufferSize];

            base.Initialize();
        }

        // Generate sound when needed
        void OnBufferNeeded(object sender, EventArgs e)
        {
            // Loop over entire buffer
            for (int i = 0; i < bufferSize - 1; i += 2)
            {
                // Calculate where we are in the wave
                double time = (double)totalTime / (double)SampleRate;
                // Generate the tone using a sine wave
                short currentSample = (short)(Math.Sin(2 * Math.PI * frequency * time) * 
                                             (double)short.MaxValue);

                // Store the generated short value in byte array
                buffer[i] = (byte)(currentSample & 0xFF);
                buffer[i + 1] = (byte)(currentSample >> 8);

                // Incrament the current time
                totalTime += 2;
            }

            // Submit the buffer for playback
            dynamicSoundEffectInstance.SubmitBuffer(buffer);
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here

            base.Draw(gameTime);
        }
    }
}
