using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;

namespace SoundEffectInstancePlayback
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        KeyboardState lastKeyboardState;

        // Store our SoundEffect resource
        SoundEffect soundEffect;
        // Instance of our SoundEffect to control playback
        SoundEffectInstance soundEffectInstance;
        // Location of the audio source
        AudioEmitter audioEmitter;
        // Location of the listener
        AudioListener audioListener;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Load the SoundEffect resource
            soundEffect = Content.Load<SoundEffect>("beep");
            soundEffectInstance = soundEffect.CreateInstance();
            audioEmitter = new AudioEmitter();
            audioListener = new AudioListener();
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            KeyboardState currentKeybaordState = Keyboard.GetState();

            // Play the sound
            if (currentKeybaordState.IsKeyDown(Keys.Space) &&
                lastKeyboardState.IsKeyUp(Keys.Space))
                soundEffectInstance.Play();
            // Pause the sound
            if (currentKeybaordState.IsKeyDown(Keys.P) &&
                lastKeyboardState.IsKeyUp(Keys.P))
                soundEffectInstance.Pause();
            // Stop
            if (currentKeybaordState.IsKeyDown(Keys.S) &&
                lastKeyboardState.IsKeyUp(Keys.S))
                soundEffectInstance.Stop();
            // Resumt
            if (currentKeybaordState.IsKeyDown(Keys.R) &&
                lastKeyboardState.IsKeyUp(Keys.R))
                soundEffectInstance.Resume();

            // Start or stop looping
            if (currentKeybaordState.IsKeyDown(Keys.L) &&
                lastKeyboardState.IsKeyUp(Keys.L))
                soundEffectInstance.IsLooped = !soundEffectInstance.IsLooped;

            // Change Pitch
            if (currentKeybaordState.IsKeyDown(Keys.Q))
                soundEffectInstance.Pitch = -1.0f;
            else if (currentKeybaordState.IsKeyDown(Keys.W))
                soundEffectInstance.Pitch = 0.0f;
            else if (currentKeybaordState.IsKeyDown(Keys.E))
                soundEffectInstance.Pitch = 1.0f;

            // Change Pan
            if (currentKeybaordState.IsKeyDown(Keys.Z))
                soundEffectInstance.Pan = -1.0f;
            else if (currentKeybaordState.IsKeyDown(Keys.X))
                soundEffectInstance.Pan = 0.0f;
            else if (currentKeybaordState.IsKeyDown(Keys.C))
                soundEffectInstance.Pan = 1.0f;

            // Change Volume
            if (currentKeybaordState.IsKeyDown(Keys.B) && 
                soundEffectInstance.Volume <= 0.99f)
                soundEffectInstance.Volume += 0.01f;
            else if (currentKeybaordState.IsKeyDown(Keys.V) &&
                soundEffectInstance.Volume >= 0.01f)
                soundEffectInstance.Volume -= 0.01f;

            // Move audio emitter
            Vector3 emitterPos = audioEmitter.Position;
            if (currentKeybaordState.IsKeyDown(Keys.Right))
                emitterPos.X += (float)gameTime.ElapsedGameTime.TotalSeconds;
            else if (currentKeybaordState.IsKeyDown(Keys.Left))
                emitterPos.X -= (float)gameTime.ElapsedGameTime.TotalSeconds;
            if (currentKeybaordState.IsKeyDown(Keys.Up))
                emitterPos.Z -= (float)gameTime.ElapsedGameTime.TotalSeconds;
            else if (currentKeybaordState.IsKeyDown(Keys.Down))
                emitterPos.Z += (float)gameTime.ElapsedGameTime.TotalSeconds;

            // Store new position
            audioEmitter.Position = emitterPos;
            // Set the audio emitter and listener values
            soundEffectInstance.Apply3D(audioListener, audioEmitter);

            lastKeyboardState = currentKeybaordState;

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here

            base.Draw(gameTime);
        }
    }
}
