using System;
using System.IO;
using System.IO.IsolatedStorage;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;

namespace IsolatedStorage
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        SpriteFont font;
        List<Vector2> points;
        int currentFrameCount = 0;
        bool isExiting = false;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            // Frame rate is 30 fps by default for Windows Phone.
            TargetElapsedTime = TimeSpan.FromTicks(333333);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // Support tap gestures to map points
            TouchPanel.EnabledGestures = GestureType.Tap;
            points = new List<Vector2>();

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            font = Content.Load<SpriteFont>("font");
            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        protected override void OnExiting(object sender, EventArgs args)
        {
            // Save our data
            using (IsolatedStorageFile file = IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (isExiting)
                {
                    file.DeleteFile("currentState");
                }
                else
                {
                    using (BinaryWriter writer = new BinaryWriter(file.CreateFile("currentState")))
                    {
                        writer.Write(currentFrameCount);
                    }
                }
                using (BinaryWriter writer = new BinaryWriter(file.CreateFile("pointData")))
                {
                    foreach (Vector2 v in points)
                    {
                        writer.Write(v.X);
                        writer.Write(v.Y);
                    }
                }
            }
            base.OnExiting(sender, args);
        }

        protected override void OnActivated(object sender, EventArgs args)
        {
            // Load the data
            using (IsolatedStorageFile file = IsolatedStorageFile.GetUserStoreForApplication())
            {
                using (BinaryReader reader = new BinaryReader(file.OpenFile("currentState",
                    FileMode.OpenOrCreate)))
                {
                    if (reader.BaseStream.Length > 0)
                        currentFrameCount = reader.ReadInt32();
                }
                using (BinaryReader reader = new BinaryReader(file.OpenFile("pointData", 
                    FileMode.OpenOrCreate)))
                {
                    for(int i = 0; i < reader.BaseStream.Length / 8; i++)
                    {
                        points.Add(new Vector2(reader.ReadSingle(), reader.ReadSingle()));
                    }
                }
            }
            base.OnActivated(sender, args);
        }
        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
            {
                isExiting = true;
                this.Exit();
            }

            // Add any taps to our points collection
            while (TouchPanel.IsGestureAvailable)
            {
                GestureSample gesture = TouchPanel.ReadGesture();
                points.Add(gesture.Position);
            }

            currentFrameCount++;
            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();
            foreach (Vector2 position in points)
            {
                spriteBatch.DrawString(font, "x", position, Color.White);
            }
            spriteBatch.DrawString(font, currentFrameCount.ToString(), Vector2.Zero, Color.Black);
            spriteBatch.DrawString(font, currentFrameCount.ToString(), Vector2.One, Color.White);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
